//+------------------------------------------------------------------+
//|                                           Schaff Trend Cycle.mq4 |
//|                                                           mladen |
//+------------------------------------------------------------------+
#property copyright "mladen"
#property link      "mladenfx@gmail.com"
#property link      "www.forex-station.com"

#property indicator_separate_window
#property indicator_buffers 2
#property indicator_minimum 0
#property indicator_maximum 1
#property strict

//
//
//

enum enTimeFrames
{
         tf_cu  = 0,                                                  // Current time frame
         tf_m1  = PERIOD_M1,                                          // 1 minute
         tf_m5  = PERIOD_M5,                                          // 5 minutes
         tf_m15 = PERIOD_M15,                                         // 15 minutes
         tf_m30 = PERIOD_M30,                                         // 30 minutes
         tf_h1  = PERIOD_H1,                                          // 1 hour
         tf_h4  = PERIOD_H4,                                          // 4 hours
         tf_d1  = PERIOD_D1,                                          // Daily
         tf_w1  = PERIOD_W1,                                          // Weekly
         tf_mn1 = PERIOD_MN1,                                         // Monthly
         tf_n1  = -1,                                                 // First higher time frame
         tf_n2  = -2,                                                 // Second higher time frame
         tf_n3  = -3,                                                 // Third higher time frame
         tf_cus = 12345678                                            // Custom time frame
      };
input enTimeFrames            inpTimeFrame       = tf_cu;             // Timeframe to use
input int                     inpTimeFrameCustom = 0;                 // Custom timeframe 
input string                  inpObjID           = "stc o1";          // Objects unique ID
extern int                    STCPeriod          = 10;                // Schaff period
extern int                    FastMAPeriod       = 23;                // Fast ema period
extern int                    SlowMAPeriod       = 50;                // Slow ema period
extern int                    HistoWidth         = 3;                 // Histogram bars width
input bool                    AutoHisto        = true;                       // Automatically adjust histo width
extern color                  UpHistoColor       = clrLimeGreen;      // Schaff sloping up histogram color
extern color                  DnHistoColor       = clrRed;            // Schaff sloping down histogram color
input string                  __arr__01          = "";                //. Arrows settings
input bool                    arrowsVisible      = false;             // Show arrows
input bool                    inpArrowsOnLast    = false;             // Arrows last bar (in mtf mode)?
input double                  arrowsDisplaceUp   = 0.5;               // Arrow gap up
input double                  arrowsDisplaceDn   = 0.5;               // Arrow gap down
input color                   arrowsUpColor      = clrBlue;           // Arrows up color
input color                   arrowsDnColor      = clrRed;            // Arrows down color
input int                     arrowsUpCode       = 233;               // Arrows up code
input int                     arrowsDnCode       = 234;               // Arrows down code
input int                     arrowsUpSize       = 2;                 // Arrows up size
input int                     arrowsDnSize       = 2;                 // Arrows down size
input string              __alr__00          = "";         //.Alerts settings
input bool                inpAlertsOn        = false;      // Alerts on?
input bool                inpAlertsOnCurrent = true;       // Alerts open bar?
input bool                inpAlertsMessage   = true;       // Alerts message?
input bool                inpAlertsSound     = false;      // Alerts sound?
input bool                inpAlertsEmail     = false;      // Alerts email?
input bool                inpAlertsPush      = false;      // Alerts notification?
//
//
//
//
//

double stcBuffer[];
double stcBufferUA[];
double stcBufferUB[];
double macdBuffer[];
double fastKBuffer[];
double fastDBuffer[];
double fastKKBuffer[];
double trend[],count[];
struct sGlobalStruct
{
   string indiFileName;
   int    indiTimeFrame;
   int    tfcustom;
   int    wBar;
   int       width;
};
sGlobalStruct glo;
#define _mtfCall(_buff,_ind) iCustom(_Symbol,glo.indiTimeFrame,glo.indiFileName,tf_cu,0,inpObjID,STCPeriod,FastMAPeriod,SlowMAPeriod,HistoWidth,AutoHisto,UpHistoColor,DnHistoColor,"",arrowsVisible,inpArrowsOnLast,arrowsDisplaceUp,arrowsDisplaceDn,arrowsUpColor,arrowsDnColor,arrowsUpCode,arrowsDnCode,arrowsUpSize,arrowsDnSize,__alr__00,inpAlertsOn,inpAlertsOnCurrent,inpAlertsMessage,inpAlertsSound,inpAlertsEmail,inpAlertsPush,_buff,_ind)


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int OnInit()
{
 if (AutoHisto)
   {
      int scale = int(ChartGetInteger(0,CHART_SCALE));
      switch(scale) 
	   {
	      case 0: glo.width =  1; break;
	      case 1: glo.width =  1; break;
		   case 2: glo.width =  3; break;
		   case 3: glo.width =  4; break;
		   case 4: glo.width =  8; break;
		   case 5: glo.width = 16; break;
	   }
	}
	else { glo.width = HistoWidth; }
	
   IndicatorBuffers(9);
   SetIndexBuffer(0,stcBufferUA); SetIndexStyle(0, DRAW_HISTOGRAM,EMPTY,glo.width,UpHistoColor);
   SetIndexBuffer(1,stcBufferUB); SetIndexStyle(1, DRAW_HISTOGRAM,EMPTY,glo.width,DnHistoColor);   
   SetIndexBuffer(2,stcBuffer);
   SetIndexBuffer(3,macdBuffer);
   SetIndexBuffer(4,fastKBuffer);
   SetIndexBuffer(5,fastDBuffer);
   SetIndexBuffer(6,fastKKBuffer);
   SetIndexBuffer(7,trend);
   SetIndexBuffer(8,count);
   
   glo.indiFileName = WindowExpertName();
   if (inpTimeFrameCustom==0) glo.tfcustom =(enTimeFrames)timeFrameValue(inpTimeFrameCustom);
   glo.indiTimeFrame = (inpTimeFrame!=tf_cus) ? (enTimeFrames)timeFrameValue(inpTimeFrame) : (enTimeFrames)inpTimeFrameCustom; 
   
   IndicatorSetString(INDICATOR_SHORTNAME,timeFrameToString(glo.indiTimeFrame)+" Schaff trend cycle ("+(string)STCPeriod+","+(string)FastMAPeriod+","+(string)SlowMAPeriod+")");
return(INIT_SUCCEEDED);
}
void OnDeinit(const int reason) 
{ 
   ObjectsDeleteAll(0,inpObjID+":");  
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
         int limit = fmin(Bars-counted_bars,Bars-1); count[0] = limit;
       if(ChartGetInteger(0,CHART_SCALE) != glo.width)  OnInit();    
         //
         //
         //
         
         if (glo.indiTimeFrame!=_Period)
         {
            limit = (int)fmax(limit,fmin(Bars-1,_mtfCall(8,0)*glo.indiTimeFrame/_Period));
            for(int i=limit; i>=0 && !_StopFlag; i--)
            {
               int y = iBarShift(NULL,glo.indiTimeFrame,Time[i]);
                  stcBufferUA[i] = _mtfCall(0,y);
                  stcBufferUB[i] = _mtfCall(1,y);              
            }
   return(0);
   }  
   
   //
   //
   //
   
   for(int i = limit; i >= 0; i--)
   {
      macdBuffer[i]    = iMA(NULL,0,FastMAPeriod,0,MODE_EMA,PRICE_CLOSE,i)-iMA(NULL,0,SlowMAPeriod,0,MODE_EMA,PRICE_CLOSE,i);
      double lowMacd   = macdBuffer[ArrayMinimum(macdBuffer,STCPeriod,i)];
      double highMacd  = macdBuffer[ArrayMaximum(macdBuffer,STCPeriod,i)]-lowMacd;
        fastKBuffer[i] = (highMacd > 0) ? 100*((macdBuffer[i]-lowMacd)/highMacd) : (i<Bars-1) ? fastKBuffer[i+1] : 0;
        fastDBuffer[i] = (i<Bars-1) ? fastDBuffer[i+1]+0.5*(fastKBuffer[i]-fastDBuffer[i+1]) : fastKBuffer[i];
               
        double lowStoch    = fastDBuffer[ArrayMinimum(fastDBuffer,STCPeriod,i)];
        double highStoch   = fastDBuffer[ArrayMaximum(fastDBuffer,STCPeriod,i)]-lowStoch;
           fastKKBuffer[i] = (highStoch > 0) ? 100*((fastDBuffer[i]-lowStoch)/highStoch) : (i<Bars-1) ? fastKKBuffer[i+1] : 0;
           stcBuffer[i]    = (i<Bars-1) ? stcBuffer[i+1]+0.5*(fastKKBuffer[i]-stcBuffer[i+1]) : fastKKBuffer[i];
           stcBufferUA[i]  = EMPTY_VALUE;
           stcBufferUB[i]  = EMPTY_VALUE;
           trend[i]        = (i<Bars-1) ? (stcBuffer[i] > stcBuffer[i+1]) ? 1 : (stcBuffer[i] < stcBuffer[i+1]) ? -1 : trend[i+1] : 0;      
           if (trend[i] == 1) stcBufferUA[i] = 1;
           if (trend[i] ==-1) stcBufferUB[i] = 1;
           
           //
           //
           //
      
           if (arrowsVisible)
           {
             string lookFor = inpObjID+":"+(string)Time[i]; ObjectDelete(lookFor);            
             if (i<Bars-1 && trend[i] != trend[i+1])
             {
               if (trend[i] == 1) drawArrow(i,arrowsUpColor,arrowsUpCode,arrowsUpSize,false); 
               if (trend[i] ==-1) drawArrow(i,arrowsDnColor,arrowsDnCode,arrowsDnSize,true); 
             }                                            
          }    
   } 
 if (inpAlertsOn)
   {
      glo.wBar = (inpAlertsOnCurrent) ? 0 : 1;
      if (trend[glo.wBar] != trend[glo.wBar+1])
      {
         if (trend[glo.wBar] == 1) doAlert(" up");
         if (trend[glo.wBar] ==-1) doAlert(" down");       
      }         
   }   
   return(0);
}

//+------------------------------------------------------------------+
//-------------------------------------------------------------------
//
//-------------------------------------------------------------------

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}
int timeFrameValue(int _tf)
{
   int add  = (_tf>=0) ? 0 : fabs(_tf);
   if (add != 0) _tf = _Period;
   int size = ArraySize(iTfTable); 
      int i =0; for (;i<size; i++) if (iTfTable[i]==_tf) break;
                                   if (i==size) return(_Period);
                                                return(iTfTable[(int)fmin(i+add,size-1)]);
}

//-------------------------------------------------------------------
//                                                                  
//-------------------------------------------------------------------

void drawArrow(int i,color theColor,int theCode,int theSize,bool tup)
{
   string name = inpObjID+":"+(string)Time[i];
   double gap  = iATR(NULL,0,20,i);   
   
      //
      //
      //
      
      datetime _aTime = Time[i];
      if (inpArrowsOnLast) _aTime = fmin(Time[i]+PeriodSeconds()-1,iTime(_Symbol,_Period,0));  
      ObjectCreate(name,OBJ_ARROW,0,_aTime,0);
         ObjectSet(name,OBJPROP_ARROWCODE,theCode);
         ObjectSet(name,OBJPROP_WIDTH,    theSize);
         ObjectSet(name,OBJPROP_COLOR,    theColor);
         if (tup)
               ObjectSet(name,OBJPROP_PRICE1,High[i] + arrowsDisplaceUp * gap);
         else  ObjectSet(name,OBJPROP_PRICE1,Low[i]  - arrowsDisplaceDn * gap);
}


void doAlert(string doWhat, int instanceNo=0)
{
   struct sAlertStruct
      {
         datetime prevTime;
         string   prevMessage;

            sAlertStruct() : prevTime(-1), prevMessage("") {};
      };
   static sAlertStruct m_array[];
   static int          m_arraySize = -1;
                   if (m_arraySize<instanceNo+1) m_arraySize = ArrayResize(m_array,instanceNo+1);
   
   //
   //
   //
   
   if (m_array[instanceNo].prevMessage != doWhat || m_array[instanceNo].prevTime != iTime(_Symbol,_Period,0)) 
      {
         m_array[instanceNo].prevMessage  = doWhat;
         m_array[instanceNo].prevTime     = iTime(_Symbol,_Period,0);

         //
         //
         //

         #ifndef _timeFrameToString
         #define _timeFrameToString(_tf) StringSubstr(EnumToString((ENUM_TIMEFRAMES)_tf),7)
         #endif
         
         string message = _timeFrameToString(_Period)+" "+_Symbol+" at "+TimeToString(TimeLocal(),TIME_SECONDS)+" STC "+doWhat;
            if (inpAlertsMessage) Alert(message);
            if (inpAlertsEmail)   SendMail(_Symbol+" STC ",message);
            if (inpAlertsPush)    SendNotification(message);
            if (inpAlertsSound)   PlaySound("alert2.wav");
      }
}